/**
 * Copyright (C) 2013 Kosmos <contact@kosmos.fr>
 *
 * Projet: blog
 * Version: 1.1-RC-7
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.misc;

import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereDate;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereTexteNonVideFormater;
import static com.univ.objetspartages.util.CritereRechercheUtil.getCritereThematique;

import java.sql.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.core.InfoBean;
import com.jsbsoft.jtf.database.OMContext;
import com.univ.objetspartages.om.AbstractRechercheExterne;
import com.univ.objetspartages.om.Libelle;
import com.univ.objetspartages.util.CritereRecherche;
import com.univ.utils.RechercheFicheHelper;

public class RechercheBlog extends AbstractRechercheExterne {

	/**
	 * Prepare le formulaire de recherche.
	 */
	@Override
	public boolean preparerRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
		boolean specific = false;
		final String objet = StringUtils.upperCase(infoBean.getString("OBJET"));
		if ("ARTICLEBLOG".equals(objet)) {
			specific = true;
		} else if ("BLOG".equals(objet)) {
			infoBean.set("LISTE_THEMATIQUES", Libelle.getListe(ctx, "04", ctx.getLocale()));
			specific = true;
		}
		return specific;
	}

	/**
	 * Traite la recherche.
	 */
	@Override
	public boolean traiterRECHERCHE(final InfoBean infoBean, final OMContext ctx) throws Exception {
		boolean rechercheEffectuee = false;
		final String objet = StringUtils.upperCase(infoBean.getString("OBJET"));
		// articleblog
		if ("ARTICLEBLOG".equals(objet)) {
			final List<CritereRecherche> criteres = new ArrayList<>();
			criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
			CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TITRE"));
			CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "KEYWORDS"));
			final String dateMiseEnLigneSaisie = infoBean.getString("DATE_MISE_EN_LIGNE");
			if (StringUtils.isNotBlank(dateMiseEnLigneSaisie)) {
				final Date dateMiseEnLigne = Formateur.parserDate(dateMiseEnLigneSaisie);
				if (Formateur.estSaisie(dateMiseEnLigne)) {
					final DateFormat formateur = new SimpleDateFormat("dd/MM/yyyy");
					final DateFormat formateurMoisAnnee = new SimpleDateFormat("MMMM yyyy");
					final Calendar calendar = new GregorianCalendar();
					calendar.setTime(dateMiseEnLigne);
					calendar.set(Calendar.DAY_OF_MONTH, 1);
					String valeurDateFormat = formateur.format(calendar.getTime());
					criteres.add(new CritereRecherche("DATE_MISE_EN_LIGNE_DEBUT", valeurDateFormat, formateurMoisAnnee.format(calendar.getTime())));
					calendar.set(Calendar.DAY_OF_MONTH, 31);
					valeurDateFormat = formateur.format(calendar.getTime());
					criteres.add(new CritereRecherche("DATE_MISE_EN_LIGNE_FIN", valeurDateFormat));
				}
			} else {
				CollectionUtils.addIgnoreNull(criteres, getCritereDate(infoBean, "DATE_DEBUT"));
				CollectionUtils.addIgnoreNull(criteres, getCritereDate(infoBean, "DATE_FIN"));
			}
			criteres.addAll(getCritereDefaut(infoBean));
			infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
			gestionAncienFront(infoBean, criteres);
			rechercheEffectuee = true;
		}
		// blog
		else if ("BLOG".equals(objet)) {
			final List<CritereRecherche> criteres = new ArrayList<>();
			criteres.add(new CritereRecherche("OBJET", objet, StringUtils.EMPTY));
			CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "TITRE"));
			CollectionUtils.addIgnoreNull(criteres, getCritereTexteNonVideFormater(infoBean, "DESCRIPTION"));
			CollectionUtils.addIgnoreNull(criteres, getCritereThematique(infoBean));
			criteres.addAll(getCritereDefaut(infoBean));
			infoBean.set(RechercheFicheHelper.ATTRIBUT_INFOBEAN_CRITERES, criteres);
			gestionAncienFront(infoBean, criteres);
			rechercheEffectuee = true;
		}
		return rechercheEffectuee;
	}

	@Override
	public List<String> getCriteresRequete(final String objet, final boolean listeIncluse) {
		final List<String> criteres = new ArrayList<>();
		if (objet.equals("BLOG")) {
			criteres.add("DATE_MISE_EN_LIGNE_DEBUT");
			criteres.add("DATE_MISE_EN_LIGNE_FIN");
			criteres.add("KEYWORDS");
		}
		if (objet.equals("ARTICLEBLOG")) {
			criteres.add("THEMATIQUE");
			criteres.add("DESCRIPTION");
		}
		return criteres;
	}
}
