/**
 * Copyright (C) 2013 Kosmos <contact@kosmos.fr>
 *
 * Projet: blog
 * Version: 1.1-RC-7
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.om;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.database.BasicPoolMgr;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.blog.sgbd.ArticleblogDB;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.univ.objetspartages.om.DiffusionSelective;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.FicheUniv;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.InfosStructure;
import com.univ.objetspartages.om.Metatag;
import com.univ.objetspartages.om.Rubrique;
import com.univ.objetspartages.om.Structure;
import com.univ.utils.ContexteUniv;
import com.univ.utils.DateUtil;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.Condition;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ClauseJoinHelper;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * Classe representant un objet articleblog.
 */
@FicheAnnotation()
public class Articleblog extends ArticleblogDB implements FicheUniv, DiffusionSelective {

	/**
	 * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
	 */
	public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
		String res = "";
		if (codes == null || codes.length() == 0) {
			return res;
		}
		final Articleblog articleblog = new Articleblog();
		articleblog.init();
		articleblog.setCtx(ctx);
		if (langue == null || langue.length() == 0) {
			langue = "0";
		}
		final StringTokenizer st = new StringTokenizer(codes, ";");
		while (st.hasMoreTokens()) {
			final String code = st.nextToken();
			// On cherche d'abord la version en ligne puis les autres versions
			int count = articleblog.selectCodeLangueEtat(code, langue, "0003");
			if (count == 0) {
				count = articleblog.selectCodeLangueEtat(code, langue, "");
			}
			if (count > 0) {
				if (res.length() > 1) {
					res += ";";
				}
				if (articleblog.nextItem()) {
					res += articleblog.getLibelleAffichable();
				}
			} else {
				res += "-";
			}
		}
		return res;
	}

	/**
	 * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
	 */
	public static Articleblog getFicheArticleblog(final OMContext ctx, final String code, String langue) throws Exception {
		Articleblog fiche = null;
		if (code != null && code.length() > 0) {
			if (langue == null || langue.length() == 0) {
				langue = "0"; // francais par defaut
			}
			fiche = new Articleblog();
			fiche.init();
			fiche.setCtx(ctx);
			int count = fiche.selectCodeLangueEtat(code, langue, "0003");
			if (count == 0) // si pas de version en ligne
			{
				count = fiche.selectCodeLangueEtat(code, langue, "");
			}
			if (count == 0) // si pas de version dans la langue demandee
			{
				count = fiche.selectCodeLangueEtat(code, "", "0003");
			}
			if (count > 0) {
				fiche.nextItem();
			}
		}
		return fiche;
	}

	/**
	 * Initialise l'objet metier.
	 */
	@Override
	public void init() {
		FicheUnivMgr.init(this);
		setIdArticleblog(new Long(0));
		setCode("");
		setLangue("0");
		setTitre("");
		setChapeau("");
		setContenu("");
		setKeywords("");
	}

	/**
	 * Renvoie le libelle a afficher.
	 */
	@Override
	public String getLibelleAffichable() {
		return getTitre();
	}

	/**
	 * Renvoie l'identifiant de la fiche.
	 */
	@Override
	public Long getIdFiche() {
		return getIdArticleblog();
	}

	/**
	 * Valorise l'identifiant de la fiche.
	 */
	@Override
	public void setIdFiche(final Long idFiche) {
		setIdArticleblog(idFiche);
	}

	/**
	 * Renvoie le contenu formate en HTML pour l'attribut contenu.
	 */
	public String getFormatedContenu() throws Exception {
		return UnivWebFmt.formaterEnHTML((ContexteUniv) ctx, getContenu());
	}

	/**
	 * Renvoie la rubrique de la fiche.
	 */
	public InfosRubriques getInfosRubrique() throws Exception {
		return Rubrique.renvoyerItemRubrique(getCodeRubrique());
	}

	/**
	 * Renvoie la structure de la fiche.
	 */
	public InfosStructure getInfosStructure() throws Exception {
		return Structure.renvoyerItemStructure(getCodeRattachement(), getLangue());
	}

	/**
	 * Renvoie les metadonnees de la fiche.
	 */
	public Metatag getMeta() throws Exception {
		return FicheUnivMgr.lireMeta(ctx, this);
	}

	/**
	 * Renvoie une chaine correspondant a la concatenation des champs texte de l'objet (utilise pour l'indexation Lucene).
	 */
	@Override
	public String getFullTextString() throws Exception {
		final String s = "" + " " + getTitre() + " " + getChapeau() + " " + getContenu() + " " + getKeywords();
		return s;
	}

	/**
	 * Duplication dans une nouvelle langue, le nouvel objet devient l'objet courant.
	 */
	@Override
	public void dupliquer(final String nouvelleLangue) throws Exception {
		// donnees a reinitialiser dans le nouvel objet
		FicheUnivMgr.dupliquer(this);
		// creation objet dans la langue demandee
		setIdFiche(new Long(0));
		setLangue(nouvelleLangue);
		add();
		// recuperation
		selectCodeLangueEtat(getCode(), nouvelleLangue, "0001");
		nextItem();
	}

	/**
	 * Traitement d'une requete sur l'objet.
	 */
	@Override
	public int traiterRequete(final String requete) throws Exception {
		// Recuperation des parametres de la requete
		final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
		final String keywords = RequeteUtil.renvoyerParametre(requete, "KEYWORDS");
		final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
		final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
		final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
		final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
		Date dateMiseEnligneDebut = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_MISE_EN_LIGNE_DEBUT"));
		Date dateMiseEnligneFin = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_MISE_EN_LIGNE_FIN"));
		if (!Formateur.estSaisie(dateMiseEnligneDebut) && !Formateur.estSaisie(dateMiseEnligneFin)) {
			dateMiseEnligneDebut = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_DEBUT"));
			dateMiseEnligneFin = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_FIN"));
		}
		final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
		final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
		final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
		return select(titre, keywords, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, nombre, dateMiseEnligneDebut, dateMiseEnligneFin);
	}

	/**
	 * Selection d'une fiche a partir de son code, de sa langue et de son etat.
	 */
	@Override
	public int selectCodeLangueEtat(final String code, final String langue, final String etatObjet) throws Exception {
		final RequeteSQL codeLangueEtatOrdreSurOrdreEtTitre = new RequeteSQL();
		final ClauseWhere whereCodeLangueEtat = ConditionHelper.whereCodeLangueEtat(code, langue, etatObjet);
		final ClauseOrderBy orderBy = new ClauseOrderBy();
		orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
		codeLangueEtatOrdreSurOrdreEtTitre.where(whereCodeLangueEtat).orderBy(orderBy);
		return select(codeLangueEtatOrdreSurOrdreEtTitre.formaterRequete());
	}

	/**
	 * Renvoie les articleblogs pour la rubrique demandee.
	 */
	@Override
	public int selectParCodeRubrique(final String codeRubrique, final String langue) throws Exception {
		final RequeteSQL codeRubriqueLangueEnLigneOrdreOrdreEtTitre = new RequeteSQL();
		final ClauseWhere whereCodeRubriqueLangueEtat = new ClauseWhere();
		if (StringUtils.isNotEmpty(codeRubrique)) {
			whereCodeRubriqueLangueEtat.setPremiereCondition(ConditionHelper.egalVarchar("T1.CODE_RUBRIQUE", codeRubrique));
		}
		if (StringUtils.isNotEmpty(langue)) {
			whereCodeRubriqueLangueEtat.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
		}
		whereCodeRubriqueLangueEtat.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", "0003"));
		final ClauseOrderBy orderBy = new ClauseOrderBy();
		orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
		codeRubriqueLangueEnLigneOrdreOrdreEtTitre.where(whereCodeRubriqueLangueEtat).orderBy(orderBy);
		return select(codeRubriqueLangueEnLigneOrdreOrdreEtTitre.formaterRequete());
	}

	/**
	 * Renvoie la liste des references a cette fiche.
	 */
	@Override
	public String getReferences() {
		return FicheUnivMgr.getReferenceParTexte(getContenu()) + FicheUnivMgr.getReferenceParJointure("structure", getCodeRattachement());
	}

	/**
	 * Selection d'une instance de l'objet Articleblog a partir de l'ensemble des criteres.
	 */
	public int select(final String titre, final String keywords, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet,
		final String codeRedacteur, final String code, final String nombre, final Date dateMiseEnligneDebut, final Date dateMiseEnligneFin) throws Exception {
		final ClauseWhere where = new ClauseWhere();
		// Traitement du parametre titre
		// Recherche �galement sur le champs chapeau
		if (StringUtils.isNotEmpty(titre)) {
			final ConditionList listeConditions = new ConditionList();
			listeConditions.setPremiereCondtion(ConditionHelper.rechercheMots("T1.TITRE", titre));
			listeConditions.or(ConditionHelper.rechercheMots("T1.CHAPEAU", titre));
			where.setPremiereCondition(listeConditions);
		}
		if (StringUtils.isNotEmpty(keywords)) {
			where.and(ConditionHelper.rechercheMots("T1.KEYWORDS", keywords));
		}
		if (StringUtils.isNotEmpty(codeRattachement)) {
			where.and(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
		}
		if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
			where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
		}
		if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
			where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
		}
		if (StringUtils.isNotEmpty(codeRedacteur)) {
			where.and(ConditionHelper.egalVarchar("T1.CODE_REDACTEUR", codeRedacteur));
		}
		if (StringUtils.isNotEmpty(code)) {
			where.and(ConditionHelper.egalVarchar("T1.CODE", code));
		}
		// Traitement du parametre metaDateMiseEnLigneStart
		if (Formateur.estSaisie(dateMiseEnligneDebut)) {
			final Condition condition = ConditionHelper.genericCondition("SUBSTRING(META.META_DATE_MISE_EN_LIGNE, 1, 10)", TypeOperande.NON_ECHAPABLE, dateMiseEnligneDebut,
				TypeOperande.DATE, Operateur.GREATER_EQUALS);
			where.and(condition);
		}
		// Traitement du parametre metaDateMiseEnLigneEnd
		if (Formateur.estSaisie(dateMiseEnligneFin)) {
			final Condition condition = ConditionHelper.genericCondition("SUBSTRING(META.META_DATE_MISE_EN_LIGNE, 1, 10)", TypeOperande.NON_ECHAPABLE, dateMiseEnligneFin,
				TypeOperande.DATE, Operateur.LESS_EQUALS);
			where.and(condition);
		}
		final RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
		final ClauseOrderBy orderBy = new ClauseOrderBy();
		orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
		requeteSelect.orderBy(orderBy);
		final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
		requeteSelect.limit(limite);
		// jointure METATAG
		if (Formateur.estSaisie(dateMiseEnligneDebut) || Formateur.estSaisie(dateMiseEnligneFin)) {
			requeteSelect.join(ClauseJoinHelper.creerJointureMetaTag(this));
		}
		return select(requeteSelect.formaterRequete());
	}

	public Blog getBlog() throws Exception {
		Blog blog = null;
		Connection connection = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			final String query = "select ID_BLOG from BLOG where ETAT_OBJET = ? and CODE = ?";
			connection = BasicPoolMgr.getConnection(false);
			stmt = connection.prepareStatement(query);
			stmt.setObject(1, "0003", Types.VARCHAR);
			stmt.setObject(2, getCodeRubrique(), Types.VARCHAR);
			rs = stmt.executeQuery();
			if (rs.next()) {
				blog = new Blog();
				blog.setCtx(ctx);
				blog.init();
				blog.setIdBlog(rs.getLong("ID_BLOG"));
				blog.retrieve();
			}
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (final SQLException e) {} finally {
					if (stmt != null) {
						try {
							stmt.close();
						} catch (final SQLException e) {} finally {
							BasicPoolMgr.releaseConnection(connection);
						}
					}
				}
			}
		}
		return blog;
	}

	public int getNombreReponses() throws SQLException {
		int nb = -1;
		Connection connection = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			final String query = "select count(*) as NB from COMMENTAIRE where CODE_PRINCIPAL = ? and ETAT_OBJET = ?";
			connection = BasicPoolMgr.getConnection(false);
			stmt = connection.prepareStatement(query);
			stmt.setObject(1, getCode() + ",LANGUE=" + getLangue() + ",TYPE=articleblog", Types.VARCHAR);
			stmt.setObject(2, "0003", Types.VARCHAR);
			rs = stmt.executeQuery();
			if (rs.next()) {
				nb = rs.getInt("NB");
			}
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (final SQLException e) {} finally {
					if (stmt != null) {
						try {
							stmt.close();
						} catch (final SQLException e) {} finally {
							BasicPoolMgr.releaseConnection(connection);
						}
					}
				}
			}
		}
		return nb;
	}
}
