/**
 * Copyright (C) 2013 Kosmos <contact@kosmos.fr>
 *
 * Projet: blog
 * Version: 1.1-RC-7
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;

import com.jsbsoft.jtf.core.LangueUtil;
import com.jsbsoft.jtf.database.BasicPoolMgr;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.Rubrique;

public abstract class RubriqueUtil {

	private static boolean mergeDynamicStyles(Map<String, String> styles, String key, String value) {
		boolean merged = false;
		if (styles.get(key) == null || !(styles.get(key).trim().length() > 0)){
			if (value != null) styles.put(key, value);
			else styles.put(key, "");
			merged = true;
		}
		return merged;
	}
	
	private static Map<String, String> getDynamicStyles(InfosRubriques rubrique, Map<String, String> styles) {
		if (rubrique != null) {
			boolean merged = false;
			merged |= mergeDynamicStyles(styles, "visuelRubrique", rubrique.getUrlBandeau());
			merged |= mergeDynamicStyles(styles, "couleurTitreRubrique", rubrique.getCouleurTitre());
			merged |= mergeDynamicStyles(styles, "couleurFondRubrique", rubrique.getCouleurFond());
			if (merged) {
				styles = getDynamicStyles(rubrique.getRubriqueMere(), styles);
			}
		}
		return styles;
	}
	
	public static Map<String, String> getDynamicStyles(InfosRubriques rubrique) {
		return getDynamicStyles(rubrique, new HashMap<String, String>());
	}
	
	public static InfosRubriques getParent(InfosRubriques rubrique, int deep) {
		InfosRubriques parent = null;
		if (rubrique != null && deep >= 0) {
			if (rubrique.getNiveau() == deep) {
				parent = rubrique;
			} else if (rubrique.getNiveau() > deep) {
				parent = getParent(rubrique.getRubriqueMere(), deep);
			}
		}
		return parent;
	}
	
	public static String getVisible(String code) {
		String codeRub = "";
		try {
			InfosRubriques rub = getVisible(Rubrique.renvoyerItemRubrique(code));
			if (rub != null) {
				codeRub = rub.getCode();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return codeRub;
	}
	
	public static InfosRubriques getVisible(InfosRubriques rubrique) {
		InfosRubriques rub = rubrique;
		if (rub != null && "HIDDEN".equals(rub.getCategorie())) {
			rub = getVisible(rub.getRubriqueMere());
		}
		return rub;
	}
	
	public static boolean exists(String code) throws Exception {
		return exists(code, Integer.toString(LangueUtil.getIndiceLocale(LangueUtil.getDefaultLocale())));
	}
	
	public static boolean exists(String code, String langue) throws Exception {
		boolean alreadyExists = false;
		Connection connection = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		try {
			String query = "select ID_RUBRIQUE from RUBRIQUE where CODE = ? and LANGUE = ?";
			connection = BasicPoolMgr.getConnection(true);
			stmt = connection.prepareStatement(query);
			stmt.setObject(1, code, Types.VARCHAR);
			stmt.setObject(2, langue, Types.VARCHAR);
			rs = stmt.executeQuery();
			if(rs.next()) {
				alreadyExists = true;
			}
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException e) {
				} finally {
					if (stmt != null) {
						try {
							stmt.close();
						} catch (SQLException e) {
						} finally {
							BasicPoolMgr.releaseConnection(connection);
						}
					}
				}
			}
		}
		return alreadyExists;
	}
	
}
